#import "XeeDelegate.h"
#import "XeeController.h"
#import "XeeDirectoryController.h"
#import "XeeImage.h"
#import "XeeQuicktimeLoader.h"
#import "XeeNSImageLoader.h"
#import "XeeGIFLoader.h"
#import "XeeImageSaver.h"
#import "XeeStrings.h"

#import <QuickTime/QuickTimeComponents.h>

@implementation XeeDelegate

-(id)init
{
	if(self=[super init])
	{
		EnterMovies();

		[XeeImage registerImageClass:[XeeGIFImage class]];
		[XeeImage registerImageClass:[XeeQuicktimeImage class]];
		[XeeImage registerImageClass:[XeeNSImage class]];

		[XeeImageSaver registerSaverClass:[XeePNGSaver class]];
		[XeeImageSaver registerSaverClass:[XeeJPEGSaver class]];
		[XeeImageSaver registerSaverClass:[XeeJP2Saver class]];
		[XeeImageSaver registerSaverClass:[XeeTIFFSaver class]];
		[XeeImageSaver registerSaverClass:[XeeGIFSaver class]];
		[XeeImageSaver registerSaverClass:[XeeBMPSaver class]];

		filesopened=NO;
	}
	return self;
}

-(void)applicationDidFinishLaunching:(NSNotification *)notification
{
	if(!filesopened) [self openDocument:self];
}

-(BOOL)application:(NSApplication *)app openFile:(NSString *)filename
{
	filesopened=YES;

	int windowmode=[[NSUserDefaults standardUserDefaults] integerForKey:@"windowOpening"];

	switch(windowmode)
	{
		case 0: // single window
		{
			NSArray *controllers=[XeeDirectoryController controllers];

			if([controllers count]>0)
			{
				XeeDirectoryController *controller=[controllers objectAtIndex:0];

				[[controller window] makeKeyAndOrderFront:self];
				return [controller loadImage:filename];
			}
		}
		break;

		case 1:
		{
			NSString *directory=[filename stringByDeletingLastPathComponent];
			NSArray *controllers=[XeeDirectoryController controllers];
			NSEnumerator *enumerator=[controllers objectEnumerator];
			XeeDirectoryController *controller;

			while(controller=[enumerator nextObject])
			{
				if([[controller directory] isEqual:directory])
				{
					[[controller window] makeKeyAndOrderFront:self];
					return [controller loadImage:filename];
				}
			}
		}
		break;

		case 2:
			// fall through
		break;
	}

	XeeDirectoryController *controller=[self newDirectoryWindow];
	return [controller loadImage:filename];
}



-(IBAction)openDocument:(id)sender
{
	NSOpenPanel *panel=[NSOpenPanel openPanel];

	[panel setCanChooseDirectories:YES];

	int res=[panel runModalForTypes:[XeeImage fileTypes]];

	if(res==NSOKButton)
	{
		[self application:[NSApplication sharedApplication] openFile:[[panel filenames] objectAtIndex:0]];
	}
}



-(void)menuNeedsUpdate:(NSMenu *)menu
{
	if(menu==openmenu) [self buildOpenMenu:menu];
	else if(menu==editmenu) [self updateEditMenu:menu];
}

-(BOOL)menuHasKeyEquivalent:(NSMenu *)menu forEvent:(NSEvent *)event target:(id *)target action:(SEL *)action
{
	return NO;
}

-(void)buildOpenMenu:(NSMenu *)menu
{
	while([menu numberOfItems]>2) [menu removeItemAtIndex:2];

	[self updateDefaultEditorItem];

	NSWindow *window=[[NSApplication sharedApplication] keyWindow];
	if(!window) return;

	NSString *filename=[[window delegate] currentFilename];
	NSArray *apps=(NSArray *)LSCopyApplicationURLsForURL((CFURLRef)[NSURL fileURLWithPath:filename],kLSRolesEditor);
	NSMenu *defmenu=[[[NSMenu alloc] init] autorelease];
	NSString *defeditor=[self defaultEditor];

	NSEnumerator *enumerator=[apps objectEnumerator];
	NSURL *appurl;

	while(appurl=[enumerator nextObject])
	{
		NSString *app=[appurl path];
		NSString *name=[[app lastPathComponent] stringByDeletingPathExtension];
		NSImage *image=[[NSWorkspace sharedWorkspace] iconForFile:app];

		if(![app isEqual:defeditor])
		{
			NSMenuItem *mainitem=[[[NSMenuItem alloc] initWithTitle:name action:@selector(launchAppFromMenu:) keyEquivalent:@""] autorelease];
			[mainitem setImage:image];
			[mainitem setRepresentedObject:app];
			[menu addItem:mainitem];
		}

		NSMenuItem *defitem=[[[NSMenuItem alloc] initWithTitle:name action:@selector(setDefaultEditorFromMenu:) keyEquivalent:@""] autorelease];
		[defitem setImage:image];
		[defitem setRepresentedObject:app];
		if([app isEqual:defeditor]) [defitem setState:NSOnState];
		[defmenu addItem:defitem];
	}

	[menu addItem:[NSMenuItem separatorItem]];

	NSMenuItem *defmenuitem=[[[NSMenuItem alloc] initWithTitle:S_SETDEFEDITOR action:@selector(launchApp:) keyEquivalent:@""] autorelease];
	[defmenuitem setSubmenu:defmenu];
	[menu addItem:defmenuitem];

	[apps release];
}

-(void)updateDefaultEditorItem
{
	NSString *defeditor=[self defaultEditor];
	NSMenuItem *item=[openmenu itemAtIndex:0];

	if(defeditor)
	{
		[item setTitle:[[defeditor lastPathComponent] stringByDeletingPathExtension]];
		[item setImage:[[NSWorkspace sharedWorkspace] iconForFile:defeditor]];
	}
	else
	{
		[item setTitle:S_NODEFEDITOR];
		[item setImage:nil];
	}
}

-(NSString *)defaultEditor
{
	NSString *defeditorid=[[NSUserDefaults standardUserDefaults] stringForKey:@"defaultEditor"];
	if(defeditorid) return [[NSWorkspace sharedWorkspace] absolutePathForAppBundleWithIdentifier:defeditorid];
	return nil;
}

-(void)setDefaultEditor:(NSString *)app
{
	[[NSUserDefaults standardUserDefaults] setObject:[[NSBundle bundleWithPath:app] bundleIdentifier] forKey:@"defaultEditor"];
}

-(IBAction)setDefaultEditorFromMenu:(id)sender
{
	[self setDefaultEditor:[sender representedObject]];
}

-(void)updateEditMenu:(NSMenu *)menu
{
	if([self xeeFocus])
	{
		[copyitem setTitle:S_COPYIMAGE];
		[pasteitem setTitle:S_SHOWCLIPBOARD];
	}
	else
	{
		[copyitem setTitle:S_COPY];
		[pasteitem setTitle:S_PASTE];
	}
}

-(void)paste:(id)sender
{
	XeeImage *image=[[XeeNSImage alloc] initWithPasteboard:[NSPasteboard generalPasteboard]];

	if(image)
	{
		XeeController *controller=[self newClipboardWindow];
		[controller setImage:image];
	}
}

-(IBAction)dummy:(id)sender { }



-(XeeDirectoryController *)newDirectoryWindow
{
	if(!directorynib) directorynib=[[NSNib alloc] initWithNibNamed:@"XeeDirectoryWindow" bundle:nil];
	return [self instantiateWindowFromNib:directorynib];
}

-(XeeController *)newClipboardWindow
{
	if(!clipboardnib) clipboardnib=[[NSNib alloc] initWithNibNamed:@"XeeClipboardWindow" bundle:nil];
	return [self instantiateWindowFromNib:clipboardnib];
}

-(id)instantiateWindowFromNib:(NSNib *)nib
{
	NSArray *objs;

	[nib instantiateNibWithOwner:self topLevelObjects:&objs];

	NSEnumerator *enumerator=[objs objectEnumerator];
	id obj;

	while(obj=[enumerator nextObject]) if([obj isKindOfClass:[XeeController class]]) break;
	if(!obj) return nil;

	XeeController *xee=obj;

	[[xee window] makeKeyAndOrderFront:self];

	return xee;
}

-(BOOL)xeeFocus
{
	NSWindow *win=[[NSApplication sharedApplication] keyWindow];
	if(!win) return YES;
	if([[win delegate] isKindOfClass:[XeeController class]]) return YES;
	return NO;
}

@end



@implementation XeeApplication

-(void)sendEvent:(NSEvent *)event
{
	if([event type]==NSScrollWheel)
	{
		NSWindow *window=[self keyWindow];
		NSPoint mouse=[NSEvent mouseLocation];

		if(NSPointInRect(mouse,[window frame])) goto keiji;

		NSEnumerator *enumerator=[[window drawers] objectEnumerator];
		NSDrawer *drawer;

		while(drawer=[enumerator nextObject])
		{
			if(NSPointInRect(mouse,[[[drawer contentView] window] frame])) goto keiji;
		}

		[window scrollWheel:event];
	}

	keiji:
	[super sendEvent:event];
}

@end
