#import "XeeDestinationList.h"
#import "XeeController.h"
#import "XeeStrings.h"

#define XeeDragDataType @"XeeDragDataType"
#define XeeDestinationUpdate @"XeeDestinationUpdate"



@implementation XeeDestinationView

-(void)dealloc
{
	[colorcolumn release];

	[super dealloc];
}

-(void)awakeFromNib
{
	[self setDataSource:[XeeDestinationListHandler defaultObject]];
	[self setDelegate:[XeeDestinationListHandler defaultObject]];
    [self registerForDraggedTypes:[NSArray arrayWithObjects:XeeDragDataType,NSFilenamesPboardType,nil]];

	[self setDoubleAction:@selector(destinationListClick:)];

	[self setMatchAlgorithm:KFSubstringMatchAlgorithm];
	[self setSearchColumnIdentifiers:[NSSet setWithObjects:@"filename"]];

	colorcolumn=[[NSTableColumn alloc] initWithIdentifier:@"color"];

	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(updateData:) name:XeeDestinationUpdate object:nil];
}

-(void)drawRow:(int)row clipRect:(NSRect)clipRect
{
	if(![self isRowSelected:row])
	{
		NSColor *col=[[self dataSource] tableView:self objectValueForTableColumn:colorcolumn row:row];

		if(col)
		{
			NSRect rect=[self rectOfRow:row];
			[col set];
			NSBezierPath *path=[NSBezierPath bezierPath];
			[path setLineCapStyle:NSRoundLineCapStyle];
			[path setLineWidth:rect.size.height-3];
			float x=rect.origin.x+10.0;
			float y=rect.origin.y+(rect.size.height/2.0);
			[path moveToPoint:NSMakePoint(x,y)];
			[path lineToPoint:NSMakePoint(x+rect.size.width-2*10.0,y)];
			[path stroke];
		}
	}
	[super drawRow:row clipRect:clipRect];
}
-(void)mouseDown:(NSEvent *)event
{
	NSPoint click=[self convertPoint:[event locationInWindow] fromView:nil];
	[self selectRowIndexes:[NSIndexSet indexSetWithIndex:[self rowAtPoint:click]] byExtendingSelection:NO];

	[super mouseDown:event];
}

-(void)menuForEvent:(NSEvent *)event
{
	NSPoint click=[self convertPoint:[event locationInWindow] fromView:nil];
	[self selectRowIndexes:[NSIndexSet indexSetWithIndex:[self rowAtPoint:click]] byExtendingSelection:NO];

	[super menuForEvent:event];
}

-(void)keyDown:(NSEvent *)event
{
	unichar c=[[event characters] characterAtIndex:0];

	if(c==13||c==3) [[NSApplication sharedApplication] sendAction:@selector(destinationListClick:) to:nil from:self];
	else [super keyDown:event];
}

-(unsigned int)draggingSourceOperationMaskForLocal:(BOOL)local
{
    return NSDragOperationDelete;
}

-(void)dragImage:(NSImage *)image at:(NSPoint)loc offset:(NSSize)offset event:(NSEvent *)event pasteboard:(NSPasteboard *)pboard source:(id)src slideBack:(BOOL)slide
{
	[super dragImage:image at:loc offset:offset event:event pasteboard:pboard source:src slideBack:NO];
	outside=NO;
	suggest_outside=NO;
}

-(NSDragOperation)draggingEntered:(id <NSDraggingInfo>)sender
{
	outside=NO;
	return [super draggingEntered:sender];
}

-(void)draggingExited:(id <NSDraggingInfo>)sender
{
	suggest_outside=YES;
	[super draggingExited:sender];
}

-(void)draggedImage:(NSImage *)draggedImage movedTo:(NSPoint)screenPoint
{
	if(suggest_outside)
	{
		outside=YES;
		suggest_outside=NO;
	}
}

-(void)draggedImage:(NSImage *)image endedAt:(NSPoint)point operation:(NSDragOperation)op
{
	if(outside&&op!=NSDragOperationEvery)
	{
		NSShowAnimationEffect(NSAnimationEffectDisappearingItemDefault,
		[[self window] convertBaseToScreen:[[self window] mouseLocationOutsideOfEventStream]],NSZeroSize,nil,nil,nil);
		[[[[NSSound alloc] initWithContentsOfFile:@"/System/Library/Components/CoreAudio.component/Contents/Resources/SystemSounds/dock/poof item off dock.aif" byReference:NO] autorelease] play];
		[[self dataSource] deleteLastRows];
	}
}

-(void)updateData:(id)notification
{
	[self reloadData];
}

-(IBAction)openInXee:(id)sender
{
	int index=[self selectedRow];
	if(index==0) return;

	NSString *filename=[[[self dataSource] destination:index-1] objectForKey:@"path"];
	NSApplication *app=[NSApplication sharedApplication];
	[[app delegate] application:app openFile:filename];
}

-(IBAction)openInFinder:(id)sender
{
	int index=[self selectedRow];
	if(index==0) return;

	NSString *filename=[[[self dataSource] destination:index-1] objectForKey:@"path"];
	[[NSWorkspace sharedWorkspace] openFile:filename];
}

-(IBAction)removeFromList:(id)sender
{
	int index=[self selectedRow];
	if(index==0) return;

	[[self dataSource] removeDestination:index-1];
	[[self dataSource] updateTables];
}

-(void)menuNeedsUpdate:(NSMenu *)menu
{
	BOOL enabled=[self selectedRow]==0?NO:YES;
	int count=[menu numberOfItems];

	for(int i=0;i<count;i++) [[menu itemAtIndex:i] setEnabled:enabled];
}

@end




@implementation XeeDestinationListHandler

-(id)init
{
	if(self=[super init])
	{
		NSArray *destarray=[[NSUserDefaults standardUserDefaults] objectForKey:@"destinations"];
		destinations=[[NSMutableArray alloc] initWithCapacity:16];
		[self addDestinations:destarray index:0];

		lastrowset=nil;
	}
	return self;
}

-(id)tableView:(NSTableView *)table objectValueForTableColumn:(NSTableColumn *)column row:(int)row
{
	NSString *ident=[column identifier];

	if(row==0)
	{
		if([ident isEqual:@"filename"]) return S_CHOOSEDIR;
		else return nil;
	}
	else
	{
		NSMutableDictionary *dict=[destinations objectAtIndex:row-1];
		return [dict objectForKey:ident];
	}
}

-(int)numberOfRowsInTableView:(NSTableView *)table
{
	return [destinations count]+1;
}

-(BOOL)tableView:(NSTableView *)table writeRowsWithIndexes:(NSIndexSet *)rows toPasteboard:(NSPasteboard*)pboard
{
	if([rows containsIndex:0]) return 0;

	[pboard declareTypes:[NSArray arrayWithObject:XeeDragDataType] owner:self];
	[pboard setString:[[destinations objectAtIndex:[rows firstIndex]-1] objectForKey:@"path"] forType:XeeDragDataType];

	[lastrowset release];
	lastrowset=[rows retain];

	return YES;
}

-(NSDragOperation)tableView:(NSTableView*)table validateDrop:(id <NSDraggingInfo>)info proposedRow:(int)row proposedDropOperation:(NSTableViewDropOperation)op
{
	NSPasteboard *pboard=[info draggingPasteboard];

	if(op!=NSTableViewDropAbove) return NSDragOperationNone;
	if(row==0) return NSDragOperationNone;

	if([[pboard types] containsObject:NSFilenamesPboardType])
	{
		NSArray *files=[pboard propertyListForType:NSFilenamesPboardType];

		NSEnumerator *enumerator=[files objectEnumerator];
		NSString *file;
		while(file=[enumerator nextObject])
		{
			FSRef ref;
			if(CFURLGetFSRef((CFURLRef)[NSURL fileURLWithPath:file],&ref))
			{
				Boolean folder,aliased;
				if(!FSResolveAliasFileWithMountFlags(&ref,TRUE,&folder,&aliased,kResolveAliasFileNoUI))
				{
					if(!folder) return NSDragOperationNone;
				}
			}
		}
	}

	return NSDragOperationEvery;
}

-(BOOL)tableView:(NSTableView*)table acceptDrop:(id <NSDraggingInfo>)info row:(int)row dropOperation:(NSTableViewDropOperation)op
{
	if(op!=NSTableViewDropAbove) return NSDragOperationNone;
	if(row==0) return NSDragOperationNone;

	NSPasteboard *pboard=[info draggingPasteboard];
	NSArray *files;
	int insindex=row-1;

	if([[pboard types] containsObject:NSFilenamesPboardType]) files=[pboard propertyListForType:NSFilenamesPboardType];
	else files=[NSArray arrayWithObject:[pboard stringForType:XeeDragDataType]];

	NSEnumerator *enumerator=[files objectEnumerator];
	NSString *file;
	while(file=[enumerator nextObject])
	{
		int remindex=[self findDestination:file];
		if(remindex!=NSNotFound)
		{
			[self removeDestination:remindex];
			if(remindex<insindex) insindex--;
		}
	}

	[self addDestinations:files index:insindex];

	[self updateTables];
	[table selectRowIndexes:[NSIndexSet indexSetWithIndex:insindex+1] byExtendingSelection:NO];
	[self saveList];

	return YES;
}

//-(float)tableView:(NSTableView *)tableView heightOfRow:(int)row

-(void)deleteLastRows
{
	[self removeDestination:[lastrowset firstIndex]-1];

	[lastrowset release];
	lastrowset=nil;

	[self updateTables];
	[self saveList];
}

-(void)suggestInsertion:(NSString *)directory
{
	if([self findDestination:directory]==NSNotFound)
	[self addDestinations:[NSArray arrayWithObject:directory] index:[destinations count]];

	[self updateTables];
	[self saveList];
}

-(void)addDestinations:(NSArray *)directories index:(int)index
{
	NSEnumerator *enumerator=[directories reverseObjectEnumerator];
	NSString *directory;
	while(directory=[enumerator nextObject])
	{
		NSString *dirname=[directory lastPathComponent];
		NSImage *icon=[[NSWorkspace sharedWorkspace] iconForFile:directory];
		NSColor *color=[NSColor clearColor];

		NSURL *url=[NSURL fileURLWithPath:directory];

		FSRef fsref;
		FSSpec fsspec;
		CFURLGetFSRef((CFURLRef)url,&fsref);
		FSGetCatalogInfo(&fsref,kFSCatInfoNone,NULL,NULL,&fsspec,NULL);

		IconRef dummyicon;
		SInt16 label;
		RGBColor rgbcol;
		Str255 labelstr;
		if(GetIconRefFromFile(&fsspec,&dummyicon,&label)==noErr)
		{
			ReleaseIconRef(dummyicon);

			if(label)
			if(GetLabel(label,&rgbcol,labelstr)==noErr)
			{
				color=[NSColor colorWithCalibratedRed:rgbcol.red/65280.0 green:rgbcol.green/65280.0 blue:rgbcol.blue/65280.0 alpha:0.15];
			}
		}
	
		[destinations insertObject:[NSDictionary dictionaryWithObjectsAndKeys:
		directory,@"path",dirname,@"filename",icon,@"icon",color,@"color",nil] atIndex:index];
	}
}

-(int)findDestination:(NSString *)directory
{
	int count=[destinations count];
	for(int i=0;i<count;i++) if([[[destinations objectAtIndex:i] objectForKey:@"path"] isEqual:directory]) return i;

	return NSNotFound;
}

-(void)removeDestination:(int)index
{
	[destinations removeObjectAtIndex:index];
}

-(NSDictionary *)destination:(int)index
{
	return [destinations objectAtIndex:index];
}

-(void)updateTables
{
	[[NSNotificationCenter defaultCenter] postNotification:[NSNotification notificationWithName:XeeDestinationUpdate object:nil]];
}

-(void)saveList
{
	NSMutableArray *array=[NSMutableArray arrayWithCapacity:[destinations count]];

	NSEnumerator *enumerator=[destinations objectEnumerator];
	NSDictionary *destination;
	while(destination=[enumerator nextObject]) [array addObject:[destination objectForKey:@"path"]];

	[[NSUserDefaults standardUserDefaults] setObject:array forKey:@"destinations"];
}

+(XeeDestinationListHandler *)defaultObject
{
	static XeeDestinationListHandler *def=nil;

	if(!def) def=[[XeeDestinationListHandler alloc] init];
	return def;
}

@end
