#import "XeeMisc.h"
#import "XeeImage.h"
#import "XeeDirectoryController.h"
#import "XeeStrings.h"



@implementation XeeCloseButton

-(id)initWithFrame:(NSRect)frame
{
	if(self=[super initWithFrame:frame])
	{
		[self setTitle:@""];
		[self setBordered:NO];
		[self setImage:[NSImage imageNamed:@"close_4"]];
		[self setAlternateImage:[NSImage imageNamed:@"close_2"]];
	}
	return self;
}

@end



@implementation XeeClipboardWindow

-(void)keyDown:(NSEvent *)event
{
	unichar c=[[event characters] characterAtIndex:0];

	if(c==27) [[NSApplication sharedApplication] sendAction:@selector(performClose:) to:nil from:self];
	else if(c=='=') [[NSApplication sharedApplication] sendAction:@selector(zoomIn:) to:nil from:self];
	else [super keyDown:event];
}

@end



@implementation XeeCollisionPanel

-(void)run:(NSWindow *)window source:(XeeImage *)src destination:(XeeImage *)dest mode:(int)mode;
{
	srcimage=[src retain];
	destimage=[dest retain];
	transfermode=mode;

	NSImage *iconimage=[[NSWorkspace sharedWorkspace] iconForFile:[destimage filename]];
	[iconimage setSize:NSMakeSize(128,128)];
	[icon setImage:iconimage];
	[titlefield setStringValue:[NSString stringWithFormat:S_FILEEXISTS,[[destimage filename] lastPathComponent]]];

	[oldsize setStringValue:[NSString stringWithFormat:@"%d",[destimage filesize]]];
	[newsize setStringValue:[NSString stringWithFormat:@"%d",[srcimage filesize]]];

	[oldformat setStringValue:[NSString stringWithFormat:@"%dx%d\n%@ %@",
	[destimage width],[destimage height],[destimage descriptiveDepth],[destimage format]]];
	[newformat setStringValue:[NSString stringWithFormat:@"%dx%d\n%@ %@",
	[srcimage width],[srcimage height],[srcimage descriptiveDepth],[srcimage format]]];

	[olddate setStringValue:[destimage descriptiveDate]];
	[newdate setStringValue:[srcimage descriptiveDate]];

	NSString *newname;
	NSString *destdir=[[destimage filename] stringByDeletingLastPathComponent];
	NSString *destname=[[[destimage filename] lastPathComponent] stringByDeletingPathExtension];
	NSString *destext=[[destimage filename] pathExtension];
	int n=1;

	do { newname=[destdir stringByAppendingPathComponent:[[NSString stringWithFormat:@"%@-%d",destname,n++] stringByAppendingPathExtension:destext]]; }
	while([[NSFileManager defaultManager] fileExistsAtPath:newname]);

	[namefield setStringValue:[newname lastPathComponent]];

	[[NSApplication sharedApplication] beginSheet:self modalForWindow:window modalDelegate:nil didEndSelector:nil contextInfo:nil];
}

-(void)cancelClick:(id)sender
{
	[[NSApplication sharedApplication] endSheet:self];
	[self orderOut:nil];

	[srcimage release];
	[destimage release];
}

-(void)renameClick:(id)sender
{
	[[NSApplication sharedApplication] endSheet:self];
	[self orderOut:nil];

	NSString *destination=[[[destimage filename] stringByDeletingLastPathComponent] stringByAppendingPathComponent:[namefield stringValue]];
	[controller attemptToTransferFile:[srcimage filename] to:destination mode:transfermode];

	[srcimage release];
	[destimage release];
}

-(void)replaceClick:(id)sender
{
	[[NSApplication sharedApplication] endSheet:self];
	[self orderOut:nil];

	[controller transferFile:[srcimage filename] to:[destimage filename] mode:transfermode];

	[srcimage release];
	[destimage release];
}


@end



@implementation XeeRenamePanel

-(void)run:(NSWindow *)window image:(XeeImage *)img
{
	image=[img retain];

	NSString *filename=[[image filename] lastPathComponent];
	[namefield setStringValue:filename];

	[[NSApplication sharedApplication] beginSheet:self modalForWindow:window modalDelegate:nil didEndSelector:nil contextInfo:nil];

	[[namefield currentEditor] setSelectedRange:NSMakeRange(0,[[filename stringByDeletingPathExtension] length])];
}

-(void)cancelClick:(id)sender
{
	[[NSApplication sharedApplication] endSheet:self];
	[self orderOut:nil];

	[image release];
}

-(void)renameClick:(id)sender
{
	[[NSApplication sharedApplication] endSheet:self];
	[self orderOut:nil];

	NSString *newname=[[[image filename] stringByDeletingLastPathComponent] stringByAppendingPathComponent:[namefield stringValue]];
	[controller renameFile:[image filename] to:newname];

	[image release];
}

@end
