#import "XeeQuicktimeLoader.h"

#import "XeeBitmapImage.h"
#import "XeeNSImageLoader.h"



@implementation XeeQuicktimeImage

-(id)initWithFile:(NSString *)name firstBlock:(NSData *)block attributes:(NSDictionary *)attributes;
{
	if(self=[super initWithFile:name firstBlock:block attributes:attributes])
	{
		gi=NULL;

		if(
			[[filename pathExtension] caseInsensitiveCompare:@"bmp"]!=NSOrderedSame
			&&[attrs fileHFSTypeCode]!='BMP '&&[attrs fileHFSTypeCode]!='BMP '
			&&[[filename pathExtension] caseInsensitiveCompare:@"tga"]!=NSOrderedSame
			&&[attrs fileHFSTypeCode]!='TPIC'
		) // explicitly exclude BMP and TGA - what the hell, Quicktime?
		{
			NSURL *url=[NSURL fileURLWithPath:filename];
			FSRef fsref;
			if(CFURLGetFSRef((CFURLRef)url,&fsref))
			{
				FSSpec fsspec;
				if(FSGetCatalogInfo(&fsref,kFSCatInfoNone,NULL,NULL,&fsspec,NULL)==noErr)
				{
					if(GetGraphicsImporterForFile(&fsspec,&gi)==noErr)
					{
						ImageDescriptionHandle desc;

						if(GraphicsImportGetImageDescription(gi,&desc)==noErr)
						{
							width=(*desc)->width;
							height=(*desc)->height;
							depth=(*desc)->depth;
							[self setFormat:[[[NSString alloc] initWithBytes:(*desc)->name+1 length:(*desc)->name[0] encoding:NSMacOSRomanStringEncoding] autorelease]];

							if(depth==32)
							{
								[self setTransparent:YES];
								[self setBackgroundColor:[NSColor whiteColor]];
							}


							DisposeHandle((Handle)desc);

							if([format isEqual:@"Photo - JPEG"]) [self setFormat:@"JPEG"]; // what the hell, Quicktime?

							nextindex=0;

							return self;
						}
					}
				}
			}
		}

		[self release];
	}

	return nil;
}

-(void)dealloc
{
	if(gi) CloseComponent(gi);

	[super dealloc];
}


static OSErr progress_func(short message,Fixed completeness,long refcon)
{
	XeeQuicktimeImage *image=(XeeQuicktimeImage *)refcon; // ow ow ow, 64-bit issues!

	if([image hasBeenStopped]) return codecAbortErr;

/*	static int i=0;
	if((i++)%32==0)
	{
		XeeBitmapImage *subimage=[image finalSubImage];
		[subimage triggerUpdateAction];
	}*/

	return noErr;
}

-(BOOL)load
{
	stop=NO;

	EnterMoviesOnThread(0);

	CGrafPtr origport;
	GDHandle origdev;
	GetGWorld(&origport,&origdev);

	unsigned long count;
	GraphicsImportGetImageCount(gi,&count);

	struct ICMProgressProcRecord progrec={progress_func,(long)self};
	GraphicsImportSetProgressProc(gi,&progrec);

	GraphicsImportSetQuality(gi,codecLosslessQuality);
	while(nextindex<count)
	{
		if(stop) break;

		if(GraphicsImportSetImageIndex(gi,nextindex+1)==noErr)
		{
			ImageDescriptionHandle desc;
			if(GraphicsImportGetImageDescription(gi,&desc)==noErr)
			{
				int framewidth=(*desc)->width;
				int frameheight=(*desc)->height;
				int framedepth=(*desc)->depth;
				DisposeHandle((Handle)desc);

				int type;

				if([format isEqual:@"JPEG"]&&[[NSUserDefaults standardUserDefaults] integerForKey:@"jpegYUV"]==1) type=XeeBitmapTypeYUV422;
				else if(framedepth==32||framedepth==16) type=XeeBitmapTypeARGB32;
				else if(framedepth>32) type=XeeBitmapTypeLuma8;
				else type=XeeBitmapTypeRGB24;

				XeeBitmapImage *image=[[[XeeBitmapImage alloc] initWithType:type width:framewidth height:frameheight] autorelease];

				if(image)
				{
					if(framedepth==32)
					{
						[image setTransparent:YES];
						[image setBackgroundColor:[NSColor whiteColor]];
					}

					[image setDepth:framedepth];

					if(GraphicsImportSetGWorld(gi,[image gworld],NULL)==noErr)
					{
//						[self addSubImage:image];

						ComponentResult res=GraphicsImportDraw(gi);
						if(res==noErr)
						{
							if(type==XeeBitmapTypeYUV422) [image fixYUVGamma]; 

							[self addSubImage:image];
							[image triggerUpdateAction];
						}
						else
						{
							if(!stop) [subimages removeAllObjects];
							break;
						}
					}
				}
			}
		}
		nextindex++;
	}

	SetGWorld(origport,origdev);

	ExitMoviesOnThread();

	if(stop) return NO;

	CloseComponent(gi);
	gi=NULL;

	if(![subimages count]) // failed to load any frames, ask XeeNSImage for help
	{
		NSImage *image=[[[NSImage alloc] initWithContentsOfFile:filename] autorelease];
		NSArray *images=[XeeNSImage convertRepresentations:[image representations]];
		if(images) [self addSubImages:images];

		[self triggerUpdateAction];

		return [subimages count]?YES:NO;
	}

	return YES;
}

-(BOOL)loaded
{
	return gi?NO:YES;
}

-(XeeBitmapImage *)finalSubImage { return [subimages objectAtIndex:[subimages count]-1]; }



+(NSArray *)fileTypes
{
	return [NSArray arrayWithObjects:
		@"jpg", // JPEG Image
		@"jpeg",
		@"jpe",
		@"'JPEG'",

		@"png", // Portable Network Graphics
		@"'PNG '", 
		@"'PNGf'", 

		@"gif", // Graphics Interchange Format
		@"'GIFf'",
		@"'GIF '",

		@"tif", // TIFF Image
		@"tiff", 
		@"'TIFF'", 

		@"bmp", // Windows Bitmap Image
		@"dib", //?
		@"'BMP '", 
		@"'BMPf'", 
		@"'BMPp'", 

		@"psd", // Adobe Photoshop Image
		@"'8BPS'",

		@"tga", // Targa Image
		@"'TPIC'", 

		@"jp2", // JPEG 2000 Image
		@"'jp2 '", 

		@"pict", // PICT Image
		@"pct", 
		@"pic",
		@"'PICT'", 

		@"fpx", // FlashPix Image
		@"'FPix'", 

		@"qtif", // Apple Quicktime Image
		@"qti",
		@"qif", 
		@"'qtif'", 

		@"sgi", // Silicon Graphics Image
		@"'SGI '", 
		@"'.SGI'", 

		@"pntg", // Apple MacPaint image
		@"'PNTG'", 

		@"mac", //?

		nil
	];
}

@end
